/*
 * 2011 Grid-Net Inc Steve Iribarne <siribarne@grid-net.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include "libusb.h"
#include "libusb_handler.h"

inline int cpu_to_le16( int in){
    int tmp = ((in & 0x000000ff)>>8)|((in&0x0000ff00)<<8);
    return tmp;
}


int transferData(struct libusb_device_handle *dev, size_t tx_size, char *data) {
	int retval = 0;
    struct _ST_HID_CBW *cbw = NULL;
	size_t writesize = 0;
    int payload_size = 0;
    int numbufs;
    unsigned int tag = 0;
    unsigned int userbufidx = 0;
    unsigned int curbuf_ref = 0;
    unsigned char *buf;
    int txlen;

    writesize = TX_MSG_SIZE_1K + 1;

    cbw = malloc(sizeof(struct _ST_HID_CBW));
    memset( cbw, 0, sizeof(struct  _ST_HID_CBW ) );
    cbw->Cdb.Command = BLTC_DOWNLOAD_FW;
    cbw->Tag = ++tag;
    cbw->cmd = 1; 
    cbw->Signature = CBW_BLTC_SIGNATURE;
    cbw->XferLength =tx_size; 
    cbw->Flags = CBW_HOST_TO_DEVICE_DIR;
    cbw->Cdb.Length = 0xf0804a00; //tx_size;

    payload_size = sizeof(struct _ST_HID_CBW);

    retval = libusb_control_transfer(dev,
                    HID_BLTC_REQUEST_TYPE,
                    HID_BLTC_REQUEST,
                    0x0200|HID_BLTC_REPORT_TYPE_COMMAND_OUT,
                    0,
                    (unsigned char *)cbw,
                    payload_size,
                    (int)2000);
    if (retval != payload_size) {
        printf("\n%s: %s\n", __func__, libusb_error_name(retval));
        return -ERR_TX;
    }
    free(cbw);

    /* find the number of buffer's to transfer */
    payload_size = writesize - 1;
    buf = malloc(writesize);

    numbufs = tx_size / payload_size;
    if(tx_size % payload_size)
        numbufs++;

    userbufidx = 0;
    for (curbuf_ref = 0; curbuf_ref < (numbufs - 1); curbuf_ref++) {
		if (!(curbuf_ref / 333)) {
			printf(".");
		}
        memcpy(buf + 1, data + userbufidx, payload_size);
        userbufidx += payload_size;
        buf[0] = HID_BLTC_REPORT_TYPE_DATA_OUT;
        retval = libusb_control_transfer(dev,
               HID_BLTC_REQUEST_TYPE,
               HID_BLTC_REQUEST,
               0x0200|HID_BLTC_REPORT_TYPE_DATA_OUT,
               0,
               buf,
               writesize,
               2000);
        if (retval != writesize) {
            printf("\n%s: buffer %d: %s", __func__, curbuf_ref, libusb_error_name(retval));
            free(buf);
            return -ERR_TX;
        }
    }
    payload_size = tx_size - userbufidx;
    memcpy(buf+1, data+userbufidx, payload_size);

    buf[0] = HID_BLTC_REPORT_TYPE_DATA_OUT;
    retval = libusb_control_transfer(dev,
             HID_BLTC_REQUEST_TYPE,
             HID_BLTC_REQUEST,
             0x0200|HID_BLTC_REPORT_TYPE_DATA_OUT,
             0,
             buf,
             writesize,
             2000);
    if (retval != writesize) {
        printf("\n%s: buffer %d: %s", __func__, curbuf_ref, libusb_error_name(retval));
        return -ERR_TX;
    }

	if ( !(retval = libusb_interrupt_transfer(dev, 1, buf, 14, &txlen, 500))) {
	    printf("\n%s: error during read: %s", __func__, libusb_error_name(retval));
	    return -ERR_TX;
	}

    free(buf);

    return SUCCESS;
}


int usb_write(int size, char *dataBuf)
{
    struct libusb_device_handle *devh = NULL;
    libusb_device *dev, **devs;
    int retval;
    ssize_t cnt;
    int i = 0;
    int status = -ERR_NO_DEV;

    libusb_init(NULL);

    cnt = libusb_get_device_list(NULL, &devs);
    if (cnt < 0)
        return (int) cnt;

    while ((dev = devs[i++]) != NULL) {
        struct libusb_device_descriptor desc;
        retval = libusb_get_device_descriptor(dev, &desc);
        if (retval < 0) {
            fprintf(stderr, "Error: Failed to get device descriptor");
            break;
        }

        if ((desc.idVendor == FREESCALE_VENDOR_ID) && (desc.idProduct == FREESCALE_PRODUCT_ID)) {
            printf("Found i.MX board. Sending bootstream.");
            retval = libusb_open(dev, &devh);
            if (retval == 0) {
                if (devh != NULL) {
                    if (libusb_claim_interface(devh, 0) != 0 ) {
                        retval = libusb_detach_kernel_driver(devh, 0);
                        if (retval != 0 ) {
                            printf("\n%s: %s\n", __func__, libusb_error_name(retval));
                            status = -ERR_DEV_BUSY;
                            break;
                        }
                        else {
                            status = transferData(devh, size, dataBuf);
                            libusb_release_interface(devh, 0);
                        }
                    }
                    else {
                        status = transferData(devh, size, dataBuf);
                        libusb_release_interface(devh, 0);
                    }

                    libusb_close(devh);
                    break;
                }
                else {
                    printf("Error: Unable to get device handle\n");
                }
            }
            else {
                printf("Error: Unable to open device.  Check permissions, or run as superuser.\n");
            }
        }
    }

    libusb_free_device_list(devs, 1);
    libusb_exit(NULL);

    return status;
}

int print_devs(void)
{
    libusb_device *dev, **devs;
    int retval = 0;
    ssize_t cnt;
    int i = 0;

    retval = libusb_init(NULL);
    if (retval < 0)
        return retval;

    cnt = libusb_get_device_list(NULL, &devs);
    if (cnt < 0)
        return (int) cnt;

    while ((dev = devs[i++]) != NULL) {
        struct libusb_device_descriptor desc;
        retval = libusb_get_device_descriptor(dev, &desc);
        if (retval < 0) {
            fprintf(stderr, "Error: Failed to get device descriptor");
            break;
        }

        printf("%04x:%04x (bus %d, device %d)\n",
            desc.idVendor, desc.idProduct,
            libusb_get_bus_number(dev), libusb_get_device_address(dev));
    }

    libusb_free_device_list(devs, 1);
    libusb_exit(NULL);

    return retval;
}
